package com.ruoyi.demo.service.impl;

import cn.hutool.core.lang.Dict;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ruoyi.demo.entity.App;
import com.ruoyi.demo.entity.DlUser;
import com.ruoyi.demo.entity.Statistics;
import com.ruoyi.demo.entity.bo.PlanBo;
import com.ruoyi.demo.entity.bo.StatisticsAddBo;
import com.ruoyi.demo.entity.vo.PlanHomeVo;
import com.ruoyi.demo.entity.vo.ScanListVo;
import com.ruoyi.demo.mapper.AppMapper;
import com.ruoyi.demo.mapper.DlUserMapper;
import com.ruoyi.demo.mapper.StatisticsMapper;
import com.ruoyi.demo.service.StatisticsService;
import com.ruoyi.demo.utils.SignUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class StatisticsServiceImpl implements StatisticsService {
    @Autowired
    StatisticsMapper statisticsMapper;
    @Autowired
    DlUserMapper dlUserMapper;
    @Autowired
    AppMapper appMapper;

    @Override
    public PageInfo<List<ScanListVo>> getScanList(PlanBo bo) {
        PageHelper.startPage(bo.getPageNum(),bo.getPageSize(),true);
        QueryWrapper<Statistics> queryWrapper = new QueryWrapper<>();
        if (bo.getStartTime() != null)
            queryWrapper.apply("`time` >= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')",bo.getStartTime());
        if (bo.getEndTime() != null)
            queryWrapper.apply("`time` <= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')",bo.getEndTime());
        List<Statistics> list = statisticsMapper.selectList(queryWrapper);
        List<ScanListVo> vos = new ArrayList<>();
        for (Statistics statistics : list){
            vos.add(new ScanListVo(statistics));
        }
        return new PageInfo(vos);
    }

    @Override
    public PlanHomeVo getScanTotal() {
        PlanHomeVo vo = new PlanHomeVo();
        vo.setTotal(statisticsMapper.selectTotal());
        vo.setTime(LocalDateTime.now(ZoneId.of("Asia/Shanghai")));
        return vo;
    }

    @Override
    public String getSign(StatisticsAddBo bo) {
        QueryWrapper<App> appQueryWrapper = new QueryWrapper<>();
        appQueryWrapper.eq("appkey",bo.getAppkey());
        App app = appMapper.selectOne(appQueryWrapper);
        if (app == null)
            return "error";
        Map<String,Object> params = Dict.create()
            .set("addrInfo",bo.getAddrInfo())
            .set("appkey",bo.getAppkey())
            .set("behavior",bo.getBehavior())
            .set("dealToolCount",bo.getDealToolCount())
            .set("dealToolPrice",bo.getDealToolPrice())
            .set("equipmentCode",bo.getEquipmentCode())
            .set("equipmentId",bo.getEquipmentId())
            .set("equipmentName",bo.getEquipmentName())
            .set("goodsId",bo.getGoodsId())
            .set("goodsName",bo.getGoodsName())
            .set("planId",bo.getPlanId())
            .set("planName",bo.getPlanName())
            .set("time",bo.getTime())
            .set("userId",bo.getUserId())
            .set("userName",bo.getUserName())
            .set("userTelephone",bo.getUserTelephone())
            .set("sign",bo.getSign());
        return SignUtil.sign(params,app.getAppSecret());
    }

    @Override
    public int add(StatisticsAddBo bo) {
        QueryWrapper<App> appQueryWrapper = new QueryWrapper<>();
        appQueryWrapper.eq("appkey",bo.getAppkey());
        App app = appMapper.selectOne(appQueryWrapper);
        if (app == null)
            return -1;
        Map<String,Object> params = Dict.create()
            .set("addrInfo",bo.getAddrInfo())
            .set("appkey",bo.getAppkey())
            .set("behavior",bo.getBehavior())
            .set("dealToolCount",bo.getDealToolCount())
            .set("dealToolPrice",bo.getDealToolPrice())
            .set("equipmentCode",bo.getEquipmentCode())
            .set("equipmentId",bo.getEquipmentId())
            .set("equipmentName",bo.getEquipmentName())
            .set("goodsId",bo.getGoodsId())
            .set("goodsName",bo.getGoodsName())
            .set("planId",bo.getPlanId())
            .set("planName",bo.getPlanName())
            .set("time",bo.getTime())
            .set("userId",bo.getUserId())
            .set("userName",bo.getUserName())
            .set("userTelephone",bo.getUserTelephone())
            .set("sign",bo.getSign());
        if (!SignUtil.verify(params,app.getAppSecret()))
            return -3;

        QueryWrapper<DlUser> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("user_id",bo.getUserId());
        DlUser user = dlUserMapper.selectOne(userQueryWrapper);
        if (user == null)
            dlUserMapper.insert(new DlUser(bo));
        else{
            user.setEndEquipmentId(bo.getEquipmentId());
            user.setEndEquipmentName(bo.getEquipmentName());
            user.setEndEquipmentCode(bo.getEquipmentCode());
            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
            LocalDateTime parse = LocalDateTime.parse(bo.getTime(), dateTimeFormatter);
            user.setEndTime(parse);
            user.setTotalPrice(user.getTotalPrice() + bo.getDealToolPrice());
            dlUserMapper.update(user,userQueryWrapper);
        }
        return statisticsMapper.insert(new Statistics(bo));
    }
}
